@@ -11,6 +11,7 @@ from typing import Any, Optional
 
 from ..helpers import QNAME_PATTERN
 from .atomic_types import AtomicTypeMeta
+from .untyped import UntypedAtomic
 
 
 class AbstractQName(metaclass=AtomicTypeMeta):
@@ -74,6 +75,8 @@ class AbstractQName(metaclass=AtomicTypeMeta):
     def __eq__(self, other: object) -> bool:
         if isinstance(other, AbstractQName):
             return self.uri == other.uri and self.local_name == other.local_name
+        elif isinstance(other, (str, UntypedAtomic)):
+            return other == self.qname
         raise TypeError("cannot compare {!r} to {!r}".format(type(self), type(other)))
 
 
