@@ -11,7 +11,7 @@
 import unittest
 import xml.etree.ElementTree as ElementTree
 
-from elementpath import *
+from elementpath import select, iter_select, Selector, XPath2Parser
 
 
 class XPathSelectorsTest(unittest.TestCase):
@@ -60,6 +60,5 @@ class XPathSelectorsTest(unittest.TestCase):
         self.assertListEqual(selector1.select(root), selector2.select(root))
 
 
-
 if __name__ == '__main__':
     unittest.main()
