@@ -1622,7 +1622,7 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
             self.assertEqual(len(self.parser.function_signatures), 36)
         elif self.parser.version == '2.0':
             self.assertEqual(len(self.parser.function_signatures), 150)
-        else:
+        elif self.parser.version == '3.0':
             self.assertEqual(len(self.parser.function_signatures), 220)
 
         for key, value in self.parser.function_signatures.items():
@@ -1630,11 +1630,15 @@ class XPath1ParserTest(xpath_test_class.XPathTestCase):
             self.assertEqual(len(key), 2)
             self.assertIsInstance(key[0], datatypes.QName)
             self.assertIsInstance(key[1], int)
-            self.assertIn(key[0].local_name, function_names)
+
+            try:
+                self.assertIn(key[0].local_name, function_names)
+            except AssertionError:
+                self.assertIn(key[0].expanded_name, function_names)
 
             if self.parser.version <= '2.0':
                 self.assertIn(key[0].namespace, XPATH_FUNCTIONS_NAMESPACE)
-            else:
+            elif self.parser.version == '3.0':
                 self.assertIn(key[0].namespace, {XPATH_FUNCTIONS_NAMESPACE,
                                                  XPATH_MATH_FUNCTIONS_NAMESPACE})
 
